!!!
!!! Problems to be solved:
!!! a) The error message mimic "PAXXXX" is problematic as developers need to 
!!!    browse through PALM every time they insert a new number. I would suggest 
!!!    to change the mimic as follows:
!!!    "PALMXXXX" - PALM core error message
!!!    "USERXXXX" - User interface error messages
!!!    "NCDFXXXX" - NetCDF error messages
!!!    "LSM_XXXX" - Land surface model error messages
!!!    etc.
!!! b) The numbering of output profiles violates the modular concept. The number of
!!!    profiles should be dynamic, depending on the modules used for the specific
!!!    run. This could be solved by having a profile counter which stores the
!!!    current number of profiles. Each module can then increase this counter and
!!!    define profiles. This basically needs rather big changes in flow_statistics.
!!!    Who can do that (I gave it up due to lack of time)?
!!! c) The calls from data_output_2d etc. to most modules are done via the 
!!!    CASE DEFAULT branch like it is done for the user interface (e.g. LSM).
!!!    However, the urban canopy module deviates from this mimic. We need to talk
!!!    about a consistent way of doing this.
!!! d) Some routines need renaming (user_* or lsm_* etc.):
!!!    - user_last_actions should be user_write_restart_data or user_write_3d_binary. 
!!!    It is also strange that write_3d_binary is called from PALM, but read_3d_binary
!!!    is not (from init_3d_model). I think we need a clean concept here.
!!!    - user_read_restart_data is called from read_3d_binary. It could thus be named
!!!                             user_read_3d_binary?
!!!



!> @file newmodule_mod.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM-4U.
!
! PALM-4U is free software: you can redistribute it and/or modify it under the 
! terms of the GNU General Public License as published by the Free Software 
! Foundation, either version 3 of the License, or (at your option) any later 
! version.
!
! PALM-4U is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 2016 Leibniz Universitaet Hannover, <your institution>
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! Initial revision
! 
! Former revisions:
! -----------------
! $Id$
!
! Authors:
! --------
! @author <List of authors that contributed to this module>
!
!
! Description:
! ------------
!> <Description of the new module>
!>
!>
!> @todo <Enter things that remain to be done>
!> @note <Enter notes on the module>
!> @bug  <Enter known bugs here>
!------------------------------------------------------------------------------!
 MODULE newmodule_mod
 

    USE control_parameters,                                                    &
        ONLY:  initializing_actions

!
!-- Load required variables from existing modules
!-- USE modulename,                                                            &
!--     ONLY: ...


    IMPLICIT NONE

!
!-- Declare all global variables within the module (alphabetical order)
!-- CHARACTER ::  global_string_a   !<
!-- CHARACTER ::  global_string_b   !<
!--
!-- INTEGER(iwp) ::  global_int_var_a   !<
!-- INTEGER(iwp) ::  global_int_var_b   !<
!--
!-- LOGICAL ::  global_flag_a   !<
!-- LOGICAL ::  global_flag_b   !<
!--
!-- REAL(wp) ::  global_real_var_a   !<
!-- REAL(wp) ::  global_real_var_b   !<
!-- ...

    SAVE


    PRIVATE

    
!
!-- Add INTERFACES that must be available to other modules (alphabetical order)
!-- PUBLIC ... 

!
!-- Add VARIABLES that must be available to other modules (alphabetical order)
!-- PUBLIC ... 

!
!-- Add PROGNOSTIC QUANTITIES that must be available to other modules (alphabetical order)
!-- PUBLIC ...


!
!-- Default procedures for all new modules (not all are necessarily required, 
!-- alphabetical order is not essential)

!
!-- PALM interfaces:
!-- Data output checks for 2D/3D data to be done in check_parameters
    INTERFACE newmodule_check_data_output
       MODULE PROCEDURE newmodule_check_data_output
    END INTERFACE newmodule_check_data_output
    
!
!-- Data output checks for profile data to be done in check_parameters
    INTERFACE newmodule_check_data_output_pr
       MODULE PROCEDURE newmodule_check_data_output_pr
    END INTERFACE newmodule_check_data_output_pr
    
!
!-- Input parameter checks to be done in check_parameters
    INTERFACE newmodule_check_parameters
       MODULE PROCEDURE newmodule_check_parameters
    END INTERFACE newmodule_check_parameters

!
!-- Averaging of 3D data for output
    INTERFACE newmodule_3d_data_averaging
       MODULE PROCEDURE newmodule_3d_data_averaging
    END INTERFACE newmodule_3d_data_averaging

!
!-- Data output of 2D quantities
    INTERFACE newmodule_data_output_2d
       MODULE PROCEDURE newmodule_data_output_2d
    END INTERFACE newmodule_data_output_2d

!
!-- Data output of 3D data
    INTERFACE newmodule_data_output_3d
       MODULE PROCEDURE newmodule_data_output_3d
    END INTERFACE newmodule_data_output_3d

!
!-- Definition of data output quantities
    INTERFACE newmodule_define_netcdf_grid
       MODULE PROCEDURE newmodule_define_netcdf_grid
    END INTERFACE newmodule_define_netcdf_grid

!
!-- Output of information to the header file
    INTERFACE newmodule_header
       MODULE PROCEDURE newmodule_header
    END INTERFACE newmodule_header
  
!
!-- Initialization actions  
    INTERFACE newmodule_init
       MODULE PROCEDURE newmodule_init
    END INTERFACE newmodule_init
 
!
!-- Initialization of arrays
    INTERFACE newmodule_init_arrays
       MODULE PROCEDURE newmodule_init_arrays
    END INTERFACE newmodule_init_arrays

!
!-- Writing of binary output for restart runs  !!! renaming?!
    INTERFACE newmodule_last_actions
       MODULE PROCEDURE newmodule_last_actions
    END INTERFACE newmodule_last_actions
    
!
!-- Reading of NAMELIST parameters
    INTERFACE newmodule_parin
       MODULE PROCEDURE newmodule_parin
    END INTERFACE newmodule_parin

!
!-- Reading of parameters for restart runs
    INTERFACE newmodule_read_restart_data
       MODULE PROCEDURE newmodule_read_restart_data
    END INTERFACE newmodule_read_restart_data

!
!-- Writing of parameters for restart runs !!! renaming?!
    INTERFACE newmodule_skip_var_list
       MODULE PROCEDURE newmodule_skip_var_list
    END INTERFACE newmodule_skip_var_list

!
!-- Swapping of time levels (required for prognostic variables)
    INTERFACE newmodule_swap_timelevel
       MODULE PROCEDURE newmodule_swap_timelevel
    END INTERFACE newmodule_swap_timelevel



!
!-- New module-specific procedure(s) (alphabetical order):
    INTERFACE newmodule_newprocedure
       MODULE PROCEDURE newmodule_newprocedure
    END INTERFACE newmodule_newprocedure

 CONTAINS

!------------------------------------------------------------------------------!
! Description:
! ------------
!> Check data output for new module.
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_check_data_output( var, unit, i, ilen, k )
 
    USE control_parameters,                                                    &
        ONLY:  data_output, message_string

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  unit     !< 
    CHARACTER (LEN=*) ::  var      !<

    INTEGER(iwp) ::  i      !<
    INTEGER(iwp) ::  ilen   !<   
    INTEGER(iwp) ::  k      !<

    SELECT CASE ( TRIM( var ) )

!
!--    e.g. output of "output_var" requires new_module = .TRUE.. In that case
!--    the unit of the output variable is set. Make sure that PAXXXX is replaced
!--    by a number according to PALMs internal counter (e.g. PA0412)
!        CASE ( 'output_var' )
!           IF (  .NOT.  new_module )  THEN
!              message_string = 'output of "' // TRIM( var ) // '" requi' //  &
!                       'res new_module = .TRUE.'
!              CALL message( 'check_parameters', 'PAXXXX', 1, 2, 0, 6, 0 )
!           ENDIF
!           unit = 'm/m3'
             
       CASE DEFAULT
          unit = 'illegal'

    END SELECT

 END SUBROUTINE newmodule_check_data_output


!------------------------------------------------------------------------------!
! Description:
! ------------
!> Check data output of profiles for new module.
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_check_data_output_pr( variable, var_count, unit, dopr_unit )
 
    USE control_parameters,                                                    &
        ONLY:  data_output_pr, message_string

    USE indices

    USE profil_parameter

    USE statistics

    IMPLICIT NONE
   
    CHARACTER (LEN=*) ::  dopr_unit   !< local value of dopr_unit
    CHARACTER (LEN=*) ::  unit        !< 
    CHARACTER (LEN=*) ::  variable    !< 
 
    INTEGER(iwp) ::  user_pr_index   !< 
    INTEGER(iwp) ::  var_count       !< 

    SELECT CASE ( TRIM( variable ) )
 
!
!-- e.g. output of "output_var". #output_var can be assigned when the initial
!-- profile at time = 0 can be output. Output is only allowed if
!-- new_module = .TRUE.. Then, the number of profiles in PALM needs to be
!-- increased and the unit is set. The numbers below must be adjusted and
!-- must match the specification on flow_statistics. Make sure that PAXXXX is 
!-- replaced by a number according to PALMs internal counter (e.g. PA0412).
!-- This numbering violates the modular concept - we need a new solution here!
!        CASE ( 'output_var', '#output_var' )
!           IF (  .NOT.  land_surface )  THEN
!              message_string = 'data_output_pr = ' //                        &
!                               TRIM( data_output_pr(var_count) ) // ' is' // &
!                               'not implemented for new_module = .FALSE.'
!              CALL message( 'check_parameters', 'PAXXXX', 1, 2, 0, 6, 0 )
!           ELSE
!              dopr_index(var_count) = 89
!              dopr_unit     = 'K'
!              hom(0:nzs-1,2,89,:)  = SPREAD( - zs, 2, statistic_regions+1 )
!              IF ( data_output_pr(var_count)(1:1) == '#' )  THEN
!                 dopr_initial_index(var_count) = 90
!                 hom(0:nzs-1,2,90,:)   = SPREAD( zu, 2, statistic_regions+1 )
!                 data_output_pr(var_count)     = data_output_pr(var_count)(2:)
!              ENDIF
!              unit = dopr_unit
!           ENDIF

       CASE DEFAULT
          unit = 'illegal'

    END SELECT

 END SUBROUTINE newmodule_check_data_output_pr
 
 
!------------------------------------------------------------------------------!
! Description:
! ------------
!> Check parameters routine for new module
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_check_parameters

    USE control_parameters,                                                    &
        ONLY:  message_string

       
    IMPLICIT NONE

!
!--    Checks go here (cf. check_parameters.f90). 
           
 END SUBROUTINE newmodule_check_parameters

!------------------------------------------------------------------------------!
!
! Description:
! ------------
!> Subroutine for averaging 3D data
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_3d_data_averaging( mode, variable )
 

    USE control_parameters

    USE indices

    USE kinds

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  mode       !< 
    CHARACTER (LEN=*) ::  variable   !< 

    INTEGER(iwp) ::  i   !< 
    INTEGER(iwp) ::  j   !< 
    INTEGER(iwp) ::  k   !< 

    IF ( mode == 'allocate' )  THEN

       SELECT CASE ( TRIM( variable ) )

!           CASE ( 'output_var_twod' )
!              IF ( .NOT. ALLOCATED( output_var_twod_av ) )  THEN
!                 ALLOCATE( output_var_twod_av(nysg:nyng,nxlg:nxrg) )
!              ENDIF
!              output_var_twod_av = 0.0_wp
! 
!           CASE ( 'output_var_threed' )
!              IF ( .NOT. ALLOCATED( output_var_threed_av ) )  THEN
!                 ALLOCATE( output_var_threed_av(nzb:nzt,nysg:nyng,nxlg:nxrg) )
!              ENDIF
!              output_var_threed_av = 0.0_wp

          CASE DEFAULT
             CONTINUE

       END SELECT

    ELSEIF ( mode == 'sum' )  THEN

       SELECT CASE ( TRIM( variable ) )

!           CASE ( 'output_var_twod' )
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    c_liq_av(j,i) = c_liq_av(j,i) + c_liq(j,i)
!                 ENDDO
!              ENDDO
! 
!           CASE ( 'output_var_threed' )
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO  k = nzb, nzt
!                       output_var_threed_av(k,j,i) = output_var_threed_av(k,j,i)&
!                                                     + output_var_threed(k,j,i)
!                    ENDDO
!                 ENDDO
!              ENDDO

          CASE DEFAULT
             CONTINUE

       END SELECT

    ELSEIF ( mode == 'average' )  THEN

       SELECT CASE ( TRIM( variable ) )

!           CASE ( 'output_var_twod' )
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    c_liq_av(j,i) = c_liq_av(j,i) / REAL( average_count_3d,     &
!                                                          KIND=wp )
!                 ENDDO
!              ENDDO
! 
!           CASE ( 'output_var_threed' )
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO  k = nzb, nzt+1
!                       output_var_threed_av(k,j,i) = output_var_threed_av(k,j,i)& 
!                       / REAL( average_count_3d, KIND=wp )
!                    ENDDO
!                 ENDDO
!              ENDDO

       END SELECT

    ENDIF

 END SUBROUTINE newmodule_3d_data_averaging


!------------------------------------------------------------------------------!
!
! Description:
! ------------
!> Subroutine defining 3D output variables
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_data_output_2d( av, variable, found, grid, mode,         &
                                      local_pf, two_d, nzb_do, nzt_do )
 
    USE indices

    USE kinds


    IMPLICIT NONE

    CHARACTER (LEN=*) ::  grid       !< 
    CHARACTER (LEN=*) ::  mode       !< 
    CHARACTER (LEN=*) ::  variable   !< 

    INTEGER(iwp) ::  av   !< 
    INTEGER(iwp) ::  i    !< 
    INTEGER(iwp) ::  j    !< 
    INTEGER(iwp) ::  k    !< 
    INTEGER(iwp) ::  nzb_do   !< 
    INTEGER(iwp) ::  nzt_do   !< 

    LOGICAL ::  found   !< 
    LOGICAL ::  two_d   !< flag parameter that indicates 2D variables (horizontal cross sections)

    REAL(wp), DIMENSION(nxlg:nxrg,nysg:nyng,nzb:nzt+1) ::  local_pf !< local   (das hier ist ein Beispiel für doxygen Zeilenumbruch)
       !< array to which output data is resorted to

    found = .TRUE.

    SELECT CASE ( TRIM( variable ) )


!        CASE ( 'output_var_twod_xy' )        ! 2d-array
!           IF ( av == 0 )  THEN
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    local_pf(i,j,nzb+1) = output_var_twod(j,i)
!                 ENDDO
!              ENDDO
!           ELSE
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    local_pf(i,j,nzb+1) = output_var_twod_av(j,i)
!                 ENDDO
!              ENDDO
!           ENDIF
! 
!           two_d = .TRUE.
!           grid = 'zu1'
! 
!        CASE ( 'output_var_threed_xy', 'output_var_threed_xz',                  &
!               'output_var_threed_yz' )
!           IF ( av == 0 )  THEN
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO k = nzb_do, nzt_do
!                       local_pf(i,j,k) = output_var_threed(k,j,i)
!                    ENDDO
!                 ENDDO
!              ENDDO
!           ELSE
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO k = nzb_do, nzt_do
!                       local_pf(i,j,k) = output_var_threed_av(k,j,i)
!                    ENDDO
!                 ENDDO
!              ENDDO
!           ENDIF
! 
!           IF ( mode == 'xy' ) grid = 'zu'
! 
       CASE DEFAULT
          found = .FALSE.
          grid  = 'none'

    END SELECT
 
 END SUBROUTINE newmodule_data_output_2d

 
!------------------------------------------------------------------------------!
!
! Description:
! ------------
!> Subroutine defining 3D output variables
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_data_output_3d( av, variable, found, local_pf )
 

    USE indices

    USE kinds


    IMPLICIT NONE

    CHARACTER (LEN=*) ::  variable   !< 

    INTEGER(iwp) ::  av   !< 
    INTEGER(iwp) ::  i    !< 
    INTEGER(iwp) ::  j    !< 
    INTEGER(iwp) ::  k    !< 

    LOGICAL ::  found   !< 

    REAL(sp), DIMENSION(nxlg:nxrg,nysg:nyng,nzb:nzt+1) ::  local_pf   !< local
       !< array to which output data is resorted to


    found = .TRUE.


    SELECT CASE ( TRIM( variable ) )


!        CASE ( 'output_var_threed' )
! 
!           IF ( av == 0 )  THEN
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO  k = nzb, nzt+1
!                       local_pf(i,j,k) = output_var_threed(k,j,i)
!                    ENDDO
!                 ENDDO
!              ENDDO
!           ELSE
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO  k = nzb, nzt+1
!                       local_pf(i,j,k) = output_var_threed_av(k,j,i)
!                    ENDDO
!                 ENDDO
!              ENDDO
!           ENDIF

       CASE DEFAULT
          found = .FALSE.

    END SELECT

 END SUBROUTINE newmodule_data_output_3d


!------------------------------------------------------------------------------!
!
! Description:
! ------------
!> Subroutine defining appropriate grid for netcdf variables.
!> It is called out from subroutine netcdf.
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_define_netcdf_grid( var, found, grid_x, grid_y, grid_z )
    
    IMPLICIT NONE

    CHARACTER (LEN=*), INTENT(OUT) ::  grid_x   !< 
    CHARACTER (LEN=*), INTENT(OUT) ::  grid_y   !< 
    CHARACTER (LEN=*), INTENT(OUT) ::  grid_z   !< 
    CHARACTER (LEN=*), INTENT(IN)  ::  var      !<
    
    LOGICAL, INTENT(OUT) ::  found   !<
    
    found  = .TRUE.

!
!-- Check for the grid
    SELECT CASE ( TRIM( var ) )

       CASE ( 'output_var_threed', 'output_var_threed_xy',                     &
              'output_var_threed_xz', 'output_var_threed_yz' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zu'

       CASE DEFAULT
          found  = .FALSE.
          grid_x = 'none'
          grid_y = 'none'
          grid_z = 'none'
    END SELECT

 END SUBROUTINE newmodule_define_netcdf_grid

 
!------------------------------------------------------------------------------!
! Description:
! ------------
!> Header output for new module
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_header ( io )


    IMPLICIT NONE

 
    INTEGER(iwp), INTENT(IN) ::  io   !< Unit of the output file
 
!
!-- Header output goes here
!-- ...

 END SUBROUTINE newmodule_header


!------------------------------------------------------------------------------!
! Description:
! ------------
!> Initialization of the new module
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_init
    
    IMPLICIT NONE

!
!-- Actions for initial runs
    IF (  TRIM( initializing_actions ) /= 'read_restart_data' )  THEN
!--    ...

!
!-- Actions for restart runs
    ELSE
!--    ...

    ENDIF

 END SUBROUTINE newmodule_init


!------------------------------------------------------------------------------!
! Description:
! ------------
!> Allocate new module arrays and define pointers if required
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_init_arrays
    

    IMPLICIT NONE

!
!-- Allocate prognostic variables (see newmodule_swap_timelevel)
#if defined( __nopointer )
!   ALLOCATE ( prog_var(nysg:nyng,nxlg:nxrg) )
!   ALLOCATE ( prog_var_p(nysg:nyng,nxlg:nxrg) )

#else
!   ALLOCATE ( prog_var_1(nysg:nyng,nxlg:nxrg) )
!   ALLOCATE ( prog_var_2(nysg:nyng,nxlg:nxrg) )
#endif

!
!-- Allocate intermediate timestep arrays
!   ALLOCATE ( tprog_var_m(nysg:nyng,nxlg:nxrg) )

!
!-- Allocate other arrays
!   ALLOCATE ( some_array(nysg:nyng,nxlg:nxrg) )


#if ! defined( __nopointer )
!
!-- Initial assignment of the pointers
!   prog_var  => prog_var_1;  prog_var_p  => prog_var_2
#endif

 END SUBROUTINE newmodule_init_arrays


!------------------------------------------------------------------------------!
! Description:
! ------------
!> Parin for &newmodule_par for new modules
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_parin

    IMPLICIT NONE

    CHARACTER (LEN=80) ::  line   !< dummy string that contains the current line of the parameter file 
       
!
!-- Add all variables that should be read via the NAMELIST environment. No
!-- further modifications should be necessary in this subroutine
!   NAMELIST /newmodule_par/       parameter_a, parameter_b
       
    line = ' '
       
!
!-- Try to find newmodule package
    REWIND ( 11 )
    line = ' '
    DO WHILE ( INDEX( line, '&newmodule_par' ) == 0 )
       READ ( 11, '(A)', END=10 )  line
    ENDDO
    BACKSPACE ( 11 )

!
!-- Read user-defined namelist
    READ ( 11, newmodule_par )

!
!-- Set flag that indicates that the new module is switched on
!-- Note that this parameter needs to be declared in modules.f90
    new_module = .TRUE.

 10 CONTINUE
       
 END SUBROUTINE newmodule_parin


!------------------------------------------------------------------------------!
! Description:
! ------------
!> Swapping of timelevels
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_swap_timelevel ( mod_count )

    IMPLICIT NONE

    INTEGER, INTENT(IN) ::  mod_count  !<

!
!-- Example for prognostic variable "prog_var"
#if defined( __nopointer )
! 
!  prog_var    = prog_var_p
! 
#else
!     
!  SELECT CASE ( mod_count )
! 
!     CASE ( 0 )
! 
!        prog_var  => prog_var_1; prog_var_p  => prog_var_2
! 
!     CASE ( 1 )
!
!        prog_var  => prog_var_2; tprog_var_p  => prog_var_1
! 
!  END SELECT
#endif

 END SUBROUTINE newmodule_swap_timelevel


!------------------------------------------------------------------------------!
! Description:
! ------------
!> This routine reads the respective restart data.
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_read_restart_data  

    
    IMPLICIT NONE
       
    CHARACTER (LEN=30) ::  variable_chr  !< dummy variable to read string
       
       
    READ ( 13 )  variable_chr
    DO  WHILE ( TRIM( variable_chr ) /= '*** end new module ***' )

       SELECT CASE ( TRIM( variable_chr ) )
          
!           CASE ( 'param1' )
!              IF ( .NOT. ALLOCATED( param1 ) )  ALLOCATE( param1(...) )
!              READ ( 13 )  param1   
          
       END SELECT
          
       READ ( 13 )  variable_chr
          
    ENDDO

 END SUBROUTINE newmodule_read_restart_data  
    

!------------------------------------------------------------------------------!
! Description:
! ------------
!> This routine writes the respective restart data.
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_last_actions

    IMPLICIT NONE
       
!      WRITE ( 14 )  'param1                            '
!      WRITE ( 14 )  param1       

       
    WRITE ( 14 )  '*** end new module ***            '
       
 END SUBROUTINE newmodule_last_actions   


!------------------------------------------------------------------------------!
! Description:
! ------------
!> Skipping of parameters from restart file (binary format).
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_skip_var_list  
           
    IMPLICIT NONE
       
    CHARACTER (LEN=1)  ::  param2         !<
    CHARACTER (LEN=30) ::  variable_chr   !<
       
    READ ( 13 )  variable_chr
    DO  WHILE ( TRIM( variable_chr ) /= '*** end new module ***' )
       READ ( 13 )  param2
    ENDDO    
       
 END SUBROUTINE newmodule_skip_var_list  


!------------------------------------------------------------------------------!
! Description:
! ------------
!> Module-specific routine for new module
!------------------------------------------------------------------------------!
 SUBROUTINE newmodule_newprocedure

!
!-- New subroutine goes here
!-- ...

 END SUBROUTINE newmodule_newprocedure


 END MODULE newmodule_mod
